#include "buffer.hh"

Buffer::Buffer() : readpos(0), toppos(0)
{
}

const Datatype *Buffer::GetBegin() const
{
	return &Buf[readpos];
}

unsigned Buffer::GetSize() const
{
	return toppos - readpos;
}

unsigned Buffer::GetFree() const
{
	unsigned ceiling = 252144;
	unsigned topfree = 0;
	if(toppos < ceiling)
		topfree += (ceiling - toppos);
	
	return topfree + readpos;
}

void Buffer::Feed(const Datatype *ptr, unsigned len)
{
	/* This function does not check the capacity. If the caller really
	 * wants to make the buffer swell, it's ok. It's recommended to use
	 * GetFree() to check how much does this want to eat, however.
	 */

	unsigned newtop = (toppos + len + 0x1FFF) & ~0x1FFF;
	
	if(newtop > Buf.size())
		Buf.resize(newtop);

	toppos = newtop;
	
	for(unsigned a=0; a<len; ++a)
		Buf[toppos+a] = ptr[a];
}

void Buffer::Eat(unsigned len)
{
	readpos += len;
	if(readpos > toppos)
		readpos = toppos;
	
	if(readpos == toppos || (toppos-readpos) < readpos/2)
	{
		SitDown();
	}
}

void Buffer::SitDown()
{
	if(readpos > 0)
	{
		Buf.erase(Buf.begin(), Buf.begin()+readpos);
		toppos -= readpos;
		readpos = 0;
	}
}
